@@ -37,7 +37,7 @@
 #include <exchange-account.h>
 #include "exchange-component.h"
 
-#define d(x) x
+#define d(x)
 
 #include <sys/time.h>
 
@@ -216,6 +216,15 @@ struct prop_info {
 
 static int num_prop_infos = sizeof(prop_info) / sizeof(prop_info[0]);
 
+static gboolean
+can_browse (EBookBackend *backend)
+{
+	return backend &&
+		e_book_backend_get_source (backend) &&
+		e_source_get_property (e_book_backend_get_source (backend), "can-browse") &&
+		strcmp (e_source_get_property (e_book_backend_get_source (backend), "can-browse"), "1") == 0;
+}
+
 static void
 book_view_notify_status (EDataBookView *view, const char *status)
 {
@@ -248,6 +257,7 @@ static GNOME_Evolution_Addressbook_CallStatus
 gal_connect (EBookBackendGAL *bl)
 {
 	EBookBackendGALPrivate *blpriv = bl->priv;
+	int ldap_error = 0;
 
 #ifdef DEBUG
 	{
@@ -268,9 +278,13 @@ gal_connect (EBookBackendGAL *bl)
 
 	g_object_ref (blpriv->gc);
 	g_mutex_lock (blpriv->ldap_lock);
-	blpriv->ldap = e2k_global_catalog_get_ldap (blpriv->gc, NULL);
+	blpriv->ldap = e2k_global_catalog_get_ldap (blpriv->gc, NULL, &ldap_error);
 	if (!blpriv->ldap) {
 		g_mutex_unlock (blpriv->ldap_lock);
+		d(printf ("%s: Cannot get ldap, error 0x%x (%s)\n", G_STRFUNC, ldap_error, ldap_err2string (ldap_error) ? ldap_err2string (ldap_error) : "Unknown error"));
+		if (ldap_error == LDAP_AUTH_METHOD_NOT_SUPPORTED)
+			return GNOME_Evolution_Addressbook_UnsupportedAuthenticationMethod;
+
 		return GNOME_Evolution_Addressbook_RepositoryOffline;
 	}
 	g_mutex_unlock (blpriv->ldap_lock);
@@ -292,7 +306,7 @@ ldap_reconnect (EBookBackendGAL *bl, EDataBookView *book_view, LDAP **ldap, int
 			book_view_notify_status (book_view, _("Reconnecting to LDAP server..."));
 
 		ldap_unbind (*ldap);
-		*ldap = e2k_global_catalog_get_ldap (bl->priv->gc, NULL);
+		*ldap = e2k_global_catalog_get_ldap (bl->priv->gc, NULL, NULL);
 		if (book_view)
 			book_view_notify_status (book_view, "");
 
@@ -313,7 +327,7 @@ gal_reconnect (EBookBackendGAL *bl, EDataBookView *book_view, int ldap_status)
 			book_view_notify_status (book_view, _("Reconnecting to LDAP server..."));
 		if (bl->priv->ldap)
 			ldap_unbind (bl->priv->ldap);
-		bl->priv->ldap = e2k_global_catalog_get_ldap (bl->priv->gc, NULL);
+		bl->priv->ldap = e2k_global_catalog_get_ldap (bl->priv->gc, NULL, NULL);
 		if (book_view)
 			book_view_notify_status (book_view, "");
 
@@ -328,7 +342,7 @@ gal_reconnect (EBookBackendGAL *bl, EDataBookView *book_view, int ldap_status)
 	}
 	else {
 		g_mutex_unlock (bl->priv->ldap_lock);
-		printf("Connected and ldap is null sigh\n");
+		d(printf("Connected and ldap is null sigh\n"));
 		return FALSE;
 	}
 }
@@ -553,7 +567,7 @@ get_contact (EBookBackend *backend,
 	EDataBookView *book_view;
 	int ldap_error;
 
-	printf("get contact\n");
+	d(printf("get contact\n"));
 	switch (bl->priv->mode) {
 	case GNOME_Evolution_Addressbook_MODE_LOCAL:
 #if ENABLE_CACHE
@@ -582,7 +596,7 @@ get_contact (EBookBackend *backend,
 
 	case GNOME_Evolution_Addressbook_MODE_REMOTE :
 #if ENABLE_CACHE
-		printf("Mode:Remote\n");
+		d(printf("Mode:Remote\n"));
 		if (bl->priv->marked_for_offline && bl->priv->file_db) {
 			EContact *contact = e_book_backend_db_cache_get_contact (bl->priv->file_db, id);
 			gchar *vcard_str ;
@@ -761,7 +775,7 @@ get_contact_list (EBookBackend *backend,
 	int ldap_error;
 	char *ldap_query;
 
-	printf("get contact list\n");
+	d(printf("get contact list\n"));
 	switch (bl->priv->mode) {
 	case GNOME_Evolution_Addressbook_MODE_LOCAL:
 #if ENABLE_CACHE
@@ -790,7 +804,7 @@ get_contact_list (EBookBackend *backend,
 
 	case GNOME_Evolution_Addressbook_MODE_REMOTE:
 #if ENABLE_CACHE
-		printf("Mode : Remote\n");
+		d(printf("Mode : Remote\n"));
 		if (bl->priv->marked_for_offline && bl->priv->file_db) {
 			GList *contacts;
 			GList *vcard_strings = NULL;
@@ -1048,6 +1062,9 @@ func_contains(ESExp *f, int argc, ESExpResult **argv, void *data)
 	} else if (!strcmp(propname, "file_as")) {
 		r = e_sexp_result_new(f, ESEXP_RES_STRING);
 		r->value.string = g_strdup_printf ("(|(displayName=%s*)(sn=%s*)(%s=%s*))", str, str, ldap_attr, str);
+	} else if (g_str_equal (ldap_attr, "displayName")) {
+		r = e_sexp_result_new(f, ESEXP_RES_STRING);
+		r->value.string = g_strdup_printf("(|(displayName=%s*)(sn=%s*)(givenName=%s*))", str, str, str);
 	} else
 		r = e_sexp_result_new(f, ESEXP_RES_UNDEFINED);
 	return r;
@@ -1183,7 +1200,8 @@ build_query (EBookBackendGAL *bl, const char *query, char **ldap_query)
 	if (!r) {
 		/* Bad query or it isn't supported */
 		*ldap_query = NULL;
-		retval = GNOME_Evolution_Addressbook_QueryRefused;
+		e_sexp_unref (sexp);
+		return GNOME_Evolution_Addressbook_QueryRefused;
 	}
 
 	if (r->type == ESEXP_RES_STRING) {
@@ -1356,7 +1374,7 @@ build_contact_from_entry (EBookBackendGAL *bl, LDAPMessage *e, GList **existing_
 	attr = ldap_first_attribute (bl->priv->ldap, e, &tber);
 	while (attr) {
 		if (!strcmp(attr, "member")) {
-			printf("It is a DL\n");
+			d(printf("It is a DL\n"));
 			is_group = TRUE;
 			ldap_memfree (attr);
 			break;
@@ -1383,7 +1401,7 @@ build_contact_from_entry (EBookBackendGAL *bl, LDAPMessage *e, GList **existing_
 			g_mutex_unlock (bl->priv->ldap_lock);
 			for (i = 0; values[i]; i ++) {
 				if (!g_ascii_strcasecmp (values [i], "groupOfNames")) {
-					printf ("groupOfNames\n");
+					d(printf ("groupOfNames\n"));
 					e_contact_set (contact, E_CONTACT_IS_LIST, GINT_TO_POINTER (TRUE));
 					e_contact_set (contact, E_CONTACT_LIST_SHOW_ADDRESSES, GINT_TO_POINTER (TRUE));
 				}
@@ -1422,13 +1440,13 @@ build_contact_from_entry (EBookBackendGAL *bl, LDAPMessage *e, GList **existing_
 						}
 						else if (info->prop_type & PROP_TYPE_GROUP) {
 							char *grpattrs[3];
-							int i, view_limit = -1, ldap_error, count;
+							int i, view_limit = -1, ldap_error = LDAP_SUCCESS, count;
 							EDataBookView *book_view;
 							LDAPMessage *result;
 							char **email_values, **cn_values, **member_info;
 
 							if (!subldap) {
-								subldap = e2k_global_catalog_get_ldap (bl->priv->gc, NULL);
+								subldap = e2k_global_catalog_get_ldap (bl->priv->gc, NULL, NULL);
 							}
 							grpattrs[0] = "cn";
 							grpattrs[1] = "mail";
@@ -1444,7 +1462,7 @@ build_contact_from_entry (EBookBackendGAL *bl, LDAPMessage *e, GList **existing_
 
 							count = ldap_count_values (values);
 							member_info = g_new0 (gchar *, count+1);
-							printf ("Fetching members\n");
+							d(printf ("Fetching members\n"));
 							for (i=0; values[i]; i++) {
 								/* get the email id for the given dn */
 								/* set base to DN and scope to base */
@@ -1591,11 +1609,10 @@ ldap_search_handler (LDAPOp *op, LDAPMessage *res)
 	int msg_type;
 
 	d(printf ("ldap_search_handler (%p)\n", view));
-	printf("%s:%s: search handler \n", G_STRLOC, G_STRFUNC);
 	g_mutex_lock (bl->priv->ldap_lock);
 	if (!bl->priv->ldap) {
 		g_mutex_unlock (bl->priv->ldap_lock);
-		printf("%s:%s: other error\n", G_STRLOC, G_STRFUNC);
+		d(printf("%s:%s: other error\n", G_STRLOC, G_STRFUNC));
 		e_data_book_view_notify_complete (view, GNOME_Evolution_Addressbook_OtherError);
 		ldap_op_finished (op);
 		return;
@@ -1634,14 +1651,16 @@ ldap_search_handler (LDAPOp *op, LDAPMessage *res)
 				   NULL, &ldap_error_msg, NULL, NULL, 0);
 		g_mutex_unlock (bl->priv->ldap_lock);
 		if (ldap_error != LDAP_SUCCESS) {
-			printf("%s:%s: error result\n", G_STRLOC, G_STRFUNC);
 			g_warning ("ldap_search_handler: %02X (%s), additional info: %s",
 				   ldap_error,
 				   ldap_err2string (ldap_error), ldap_error_msg);
 		}
 		ldap_memfree (ldap_error_msg);
 
-		if (ldap_error == LDAP_TIMELIMIT_EXCEEDED)
+		if ((ldap_error == LDAP_TIMELIMIT_EXCEEDED || ldap_error == LDAP_SIZELIMIT_EXCEEDED) && can_browse ((EBookBackend *)bl))
+			/* do not complain when search limit exceeded for browsable LDAPs */
+			e_data_book_view_notify_complete (view, GNOME_Evolution_Addressbook_Success);
+		else if (ldap_error == LDAP_TIMELIMIT_EXCEEDED)
 			e_data_book_view_notify_complete (view, GNOME_Evolution_Addressbook_SearchTimeLimitExceeded);
 		else if (ldap_error == LDAP_SIZELIMIT_EXCEEDED)
 			e_data_book_view_notify_complete (view, GNOME_Evolution_Addressbook_SearchSizeLimitExceeded);
@@ -1649,7 +1668,6 @@ ldap_search_handler (LDAPOp *op, LDAPMessage *res)
 			e_data_book_view_notify_complete (view, GNOME_Evolution_Addressbook_Success);
 		else
 			e_data_book_view_notify_complete (view, GNOME_Evolution_Addressbook_OtherError);
-		printf("%s:%s: o/p %d %d\n", G_STRLOC, G_STRFUNC, ldap_error, LDAP_SUCCESS);
 		ldap_op_finished (op);
 	}
 	else {
@@ -1668,7 +1686,7 @@ ldap_search_dtor (LDAPOp *op)
 	d(printf ("ldap_search_dtor (%p)\n", search_op->view));
 
 	/* unhook us from our EDataBookView */
-	printf ("ldap_search_dtor: Setting null inplace of %p in view %p\n", op, search_op->view);
+	d(printf ("ldap_search_dtor: Setting null inplace of %p in view %p\n", op, search_op->view));
 	g_object_set_data (G_OBJECT (search_op->view), "EBookBackendGAL.BookView::search_op", NULL);
 
 	bonobo_object_unref (search_op->view);
@@ -1716,7 +1734,7 @@ start_book_view (EBookBackend  *backend,
 	int view_limit;
 	GList *l;
 
-	printf("start book view\n");
+	d(printf("start book view\n"));
 	switch (bl->priv->mode) {
 	case GNOME_Evolution_Addressbook_MODE_LOCAL:
 #if ENABLE_CACHE
@@ -1744,14 +1762,21 @@ start_book_view (EBookBackend  *backend,
 #endif
 	case GNOME_Evolution_Addressbook_MODE_REMOTE:
 #if ENABLE_CACHE
-		printf("Mode:Remote\n");
+		d(printf("Mode:Remote\n"));
 		if (bl->priv->marked_for_offline && bl->priv->file_db) {
 			const char *query = e_data_book_view_get_card_query (view);
 			GPtrArray *ids = NULL;
-			printf("Marked for offline and cache present\n");
+			d(printf("Marked for offline and cache present\n"));
 
 			status = build_query (bl, e_data_book_view_get_card_query (view),
 					      &ldap_query);
+
+			/* search for anything */
+			if (!ldap_query && (status == GNOME_Evolution_Addressbook_QueryRefused || status == GNOME_Evolution_Addressbook_Success) && can_browse ((EBookBackend *)bl)) {
+				ldap_query = g_strdup ("(mail=*)");
+				status = GNOME_Evolution_Addressbook_Success;
+			}
+
 			if (status != GNOME_Evolution_Addressbook_Success || !ldap_query) {
 				e_data_book_view_notify_complete (view, status);
 				if (ldap_query)
@@ -1761,7 +1786,7 @@ start_book_view (EBookBackend  *backend,
 
 			if (bl->priv->is_summary_ready &&
 			    e_book_backend_summary_is_summary_query (bl->priv->summary, query)) {
-				printf("Summary ready and summary_query, searching from summary \n");
+				d(printf("Summary ready and summary_query, searching from summary \n"));
 				ids = e_book_backend_summary_search (bl->priv->summary, query);
 				if (ids && ids->len > 0) {
 					get_contacts_from_cache (bl, query, ids, view);
@@ -1786,12 +1811,17 @@ start_book_view (EBookBackend  *backend,
 		}
 		else {
 #endif
-			printf("Not marked for offline or cache not there\n");
+			if (!bl->priv->connected) {
+				e_data_book_view_notify_complete (view, GNOME_Evolution_Addressbook_RepositoryOffline);
+				return;
+			}
+
+			d(printf("Not marked for offline or cache not there\n"));
 			g_mutex_lock (bl->priv->ldap_lock);
 			if (!bl->priv->ldap) {
 				g_mutex_unlock (bl->priv->ldap_lock);
 				if (!gal_reconnect (bl, view, 0)) {
-					printf("%s:%s: no ldap :(\n", G_STRLOC, G_STRFUNC);
+					d(printf("%s:%s: no ldap :(\n", G_STRLOC, G_STRFUNC));
 					e_data_book_view_notify_complete (view,
 									  GNOME_Evolution_Addressbook_InvalidQuery);
 					return;
@@ -1812,12 +1842,19 @@ start_book_view (EBookBackend  *backend,
 
 			status = build_query (bl, e_data_book_view_get_card_query (view),
 					      &ldap_query);
-			printf("%s:%s: %s\n", G_STRLOC, G_STRFUNC, ldap_query);
+
+			/* search for anything */
+			if (!ldap_query &&  (status == GNOME_Evolution_Addressbook_QueryRefused || status == GNOME_Evolution_Addressbook_Success) && can_browse ((EBookBackend *)bl)) {
+				ldap_query = g_strdup ("(mail=*)");
+				status = GNOME_Evolution_Addressbook_Success;
+			}
+
+			d(printf("%s:%s: %s\n", G_STRLOC, G_STRFUNC, ldap_query ? ldap_query : "No ldap_query produced!"));
 			if (status != GNOME_Evolution_Addressbook_Success || !ldap_query) {
 				e_data_book_view_notify_complete (view, status);
 				if (ldap_query)
 					g_free (ldap_query);
-				printf("%s:%s: failure \n", G_STRLOC, G_STRFUNC);
+				d(printf("%s:%s: failure \n", G_STRLOC, G_STRFUNC));
 				return;
 			}
 
@@ -1828,7 +1865,7 @@ start_book_view (EBookBackend  *backend,
 					book_view_notify_status (view, _("Searching..."));
 
 					g_mutex_lock (bl->priv->ldap_lock);
-					printf("%s:%s: starting \n", G_STRLOC, G_STRFUNC);
+					d(printf("%s:%s: starting \n", G_STRLOC, G_STRFUNC));
 					ldap_err = ldap_search_ext (bl->priv->ldap, LDAP_ROOT_DSE,
 								    LDAP_SCOPE_SUBTREE,
 								    ldap_query,
@@ -1839,7 +1876,7 @@ start_book_view (EBookBackend  *backend,
 								    view_limit,
 								    &search_msgid);
 					g_mutex_unlock (bl->priv->ldap_lock);
-					printf("%s:%s: %d\n", G_STRLOC, G_STRFUNC, ldap_err);
+					d(printf("%s:%s: %d\n", G_STRLOC, G_STRFUNC, ldap_err));
 				} else {
 					g_mutex_unlock (bl->priv->ldap_lock);
 					bl->priv->connected = FALSE;
@@ -1849,12 +1886,12 @@ start_book_view (EBookBackend  *backend,
 			g_free (ldap_query);
 
 			if (ldap_err != LDAP_SUCCESS) {
-				printf("%s:%s: error\n", G_STRLOC, G_STRFUNC);
+				d(printf("%s:%s: error\n", G_STRLOC, G_STRFUNC));
 				book_view_notify_status (view, ldap_err2string(ldap_err));
 				return;
 			}
 			else if (search_msgid == -1) {
-				printf("%s:%s: error\n", G_STRLOC, G_STRFUNC);
+				d(printf("%s:%s: error\n", G_STRLOC, G_STRFUNC));
 				book_view_notify_status (view,
 							 _("Error performing search"));
 				return;
@@ -1863,7 +1900,6 @@ start_book_view (EBookBackend  *backend,
 				LDAPSearchOp *op = g_new0 (LDAPSearchOp, 1);
 
 				d(printf ("adding search_op (%p, %d)\n", view, search_msgid));
-				printf("%s:%s: adding search \n", G_STRLOC, G_STRFUNC);
 				op->view = view;
 				op->aborted = FALSE;
 
@@ -1872,7 +1908,7 @@ start_book_view (EBookBackend  *backend,
 				ldap_op_add ((LDAPOp*)op, E_BOOK_BACKEND (bl), NULL, view,
 					     0, search_msgid,
 					     ldap_search_handler, ldap_search_dtor);
-				printf("start_book_view: Setting op %p in book %p\n", op, view);
+				d(printf("start_book_view: Setting op %p in book %p\n", op, view));
 				g_object_set_data (G_OBJECT (view), "EBookBackendGAL.BookView::search_op", op);
 			}
 #if ENABLE_CACHE
@@ -1890,7 +1926,7 @@ stop_book_view (EBookBackend  *backend,
 	d(printf ("stop_book_view (%p)\n", view));
 
 	op = g_object_get_data (G_OBJECT (view), "EBookBackendGAL.BookView::search_op");
-	printf("STOP BOOK VIEW: Getting op %p from view %p\n", op, view);
+	d(printf("STOP BOOK VIEW: Getting op %p from view %p\n", op, view));
 	if (op) {
 		op->aborted = TRUE;
 		ldap_op_finished ((LDAPOp*)op);
@@ -2034,7 +2070,7 @@ parse_page_control(
 		morePagedResults = 0;
 	}
 	if (cookie->bv_len>0) {
-		printf("\n");
+		d(printf ("\n"));
 	}
 	else {
 		morePagedResults = 0;
@@ -2139,7 +2175,7 @@ generate_cache (EBookBackendGAL *book_backend_gal)
 	char t[15];
 	LDAPControl c[6];
 
-	printf ("Generate Cache\n");
+	d(printf ("Generate Cache\n"));
 	priv = book_backend_gal->priv;
 
 
@@ -2163,7 +2199,7 @@ getNextPage:
 		if ( ber_flatten2( prber, &c[i].ldctl_value, 0 ) == -1 ) {
 			return;
 		}
-		printf ("Setting parameters		\n");
+		d(printf ("Setting parameters		\n"));
 		c[i].ldctl_oid = LDAP_CONTROL_PAGEDRESULTS;
 		c[i].ldctl_iscritical = pagedResults > 1;
 		i++;
@@ -2188,11 +2224,11 @@ getNextPage:
 	/* loop to get the next set of entries */
 
 	if ( (pageSize !=0 ) && (morePagedResults != 0)) {
-		printf ("Start next iteration\n");
+		d(printf ("Start next iteration\n"));
 		goto getNextPage;
 	}
 	else
-		printf ("All the entries fetched and finished building the cache\n");
+		d(printf ("All the entries fetched and finished building the cache\n"));
 
 	/* Set the cache to populated and thaw the changes */
 
@@ -2247,7 +2283,7 @@ authenticate_user (EBookBackend *backend,
 		if (!exchange_account_get_context (account)) {
 			exchange_account_set_online (account);
 			if(!exchange_account_connect (account, password, &result)) {
-				printf("%s:%s: failed\n", G_STRLOC, G_STRFUNC);
+				d(printf("%s:%s: failed\n", G_STRLOC, G_STRFUNC));
 				e_data_book_respond_authenticate_user (book, opid, GNOME_Evolution_Addressbook_AuthenticationFailed);
 				return;
 			}
@@ -2265,7 +2301,7 @@ authenticate_user (EBookBackend *backend,
 				int diff;
 
 				char *t = e_book_backend_db_cache_get_time (be->priv->file_db);
-				printf("Cache is populated, check if refresh is required \n");
+				d(printf("Cache is populated, check if refresh is required \n"));
 				if (t && *t)
 					t1 = atoi (t);
 				else
@@ -2274,14 +2310,14 @@ authenticate_user (EBookBackend *backend,
 				diff = interval * 24 * 60 *60;
 				/* We have a day specified, then we cache it. */
 				if (!diff || t2 - t1 > diff) {
-					printf ("Cache older than specified period, refreshing \n");
+					d(printf ("Cache older than specified period, refreshing \n"));
 					generate_cache (be);
 				}
 				else
 					be->priv->is_summary_ready= TRUE;
 			}
 			else {
-				printf("Cache not there, generate cache\n");
+				d(printf("Cache not there, generate cache\n"));
 				generate_cache(be);
 			}
 		}
@@ -2417,7 +2453,7 @@ get_supported_auth_methods (EBookBackend *backend,
 			    guint32       opid)
 
 {
-	printf("%s:%s: NONE\n", G_STRLOC, G_STRFUNC);
+	d(printf("%s:%s: NONE\n", G_STRLOC, G_STRFUNC));
 	e_data_book_respond_get_supported_auth_methods (book,
 							opid,
 							GNOME_Evolution_Addressbook_Success,
@@ -2488,10 +2524,10 @@ load_source (EBookBackend *backend,
 
 		return GNOME_Evolution_Addressbook_RepositoryOffline;
 	}
-		printf("offlin==============\n");
+		d(printf("offlin==============\n"));
 #if ENABLE_CACHE
  	if (bl->priv->marked_for_offline) {
- 		printf("offlin==============\n");
+ 		d(printf("offlin==============\n"));
  		bl->priv->summary_file_name = g_build_filename (g_get_home_dir(), ".evolution/cache/addressbook" , uri, book_name, NULL);
  		bl->priv->summary_file_name = g_build_filename (bl->priv->summary_file_name, "cache.summary", NULL);
  		bl->priv->summary = e_book_backend_summary_new (bl->priv->summary_file_name,
@@ -2623,7 +2659,10 @@ remove_gal (EBookBackend *backend, EDataBook *book, guint32 opid)
 static char *
 get_static_capabilities (EBookBackend *backend)
 {
-	return g_strdup("net");
+	if (can_browse (backend))
+		return g_strdup ("net,do-initial-query");
+	else
+		return g_strdup ("net");
 }
 
 /**
